//
// r a w 2 g b a . c
//
// Convert a RAW image file to GBA format
//
// Apr. 4, 2003 by W.Nishida (http://www.skyfree.org)
//

#include <stdio.h>	// printf(), fopen(), fgetc(), fputc(), fclose()

int main(int argc, char** argv) {
  FILE *src, *dst;
  int r, g, b, gba;

  if (argc != 3) {
    printf("Usage: raw2gba source_file destination_file\n");
    return 1;
   }

  // Open source and destination files

  src = fopen(argv[ 1 ], "r");
  if (src == NULL) {
    printf("Source file open failure\n");
    return 1;
   }

  dst = fopen(argv[ 2 ], "w");
  if (dst == NULL) {
    printf("Festination file open failure\n");
    return 1;
   }

  // Transfer image data to VRAM frame buffer

  while (1) {
    // Read RGB data from source

    r = fgetc(src);			// Read 8-bits RED data
    if (r == EOF) break;
    g = fgetc(src);			// Read 8-bits GREEN data
    if (g == EOF) goto error;
    b = fgetc(src);			// Read 8-bits BLUE data
    if (b == EOF) goto error;

    // Convert RGB data to GBA format
    // MSB: 0BBBBBGGGGGRRRRR :LSB

    gba  = r >> 3;			// Setup lower 5-bits RED data
    gba += (g >> 3) << 5;		// Ored with 5-bits middle GREEN data
    gba += (b >> 3) << 10;		// Ored with 5-bits upper BLUE data

    // Write down to dest file (little endian)

    if (fputc(gba & 0xFF, dst) == EOF) {
      printf("File write failure\n");
      goto error;
     }

    if (fputc(gba >> 8, dst) == EOF) {
      printf("File write failure\n");
      goto error;
     }
   }

  // Closs and exit

  fclose(src);
  fclose(dst);
  return 0;

error:
  fclose(src);
  fclose(dst);
  return 1;
 }
